<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:j2="http://jhove2.org/xsd/1.0.0"
	xmlns:containerMD="http://bibnum.bnf.fr/ns/containerMD-v1" xmlns:set="http://exslt.org/sets"
	xmlns:wrapper="java:fr.bnf.spar.ingest.utility.ContainerMDWrapper"
	xmlns:exsl="http://exslt.org/common" xmlns:saxon="http://icl.com/saxon"
	extension-element-prefixes="saxon set exsl wrapper"
	exclude-result-prefixes="j2 exsl saxon set wrapper">
	<xsl:output method="xml" version="1.0" omit-xml-declaration="yes" indent="yes"/>
	<!-- Variables initialization -->
	<xsl:variable name="ArcModuleNode"
		select=".//j2:features/j2:feature[@ftid='http://jhove2.org/terms/reportable/org/jhove2/module/format/arc/ArcModule'][1]/j2:features/j2:feature"/>
	<xsl:variable name="GzipMemberSourceNodes"
		select=".//j2:feature[@ftid='http://jhove2.org/terms/reportable/org/jhove2/core/source/GzipMemberSource'][1]"/>
	<xsl:variable name="VersionBlockNode"
		select=".//j2:features/j2:feature[@ftid='http://jhove2.org/terms/reportable/org/jhove2/core/source/VersionBlockSource'][1]/j2:features/j2:feature"/>
	<xsl:variable name="MimeType1"
		select="exsl:node-set($GzipMemberSourceNodes)
		/j2:features/j2:feature[@name='PresumptiveFormats']
		/j2:features/j2:feature[@name='PresumptiveFormat']
		/j2:features/j2:feature[@name='NativeIdentifier' and j2:features/j2:feature[@name='Namespace']/j2:value='MIME']
		/j2:features/j2:feature[@name='Value']
		/j2:value"/>
	<xsl:variable name="MimeType2"
		select="/j2:jhove2/j2:feature[@name='FileSource']/j2:features/j2:feature[@name='PresumptiveFormats']
		/j2:features/j2:feature[@name='PresumptiveFormat']
		/j2:features/j2:feature[@name='NativeIdentifier' and j2:features/j2:feature[@name='Namespace']/j2:value='MIME']
		/j2:features/j2:feature[@name='Value']
		/j2:value"/>
	<xsl:variable name="wrapperInstance" select="wrapper:new()"/>
	<xsl:template match="/j2:jhove2/j2:feature[@name='FileSource']/j2:features">
		<!-- CONTAINER MD -->
		<!-- For debug purpose
			<xsl:element name="containerMD:profile">
				<xsl:value-of select="$MimeType1" />
				<xsl:text>.</xsl:text>
				<xsl:value-of select="$MimeType2" />
				</xsl:element>  -->
		<xsl:element name="containerMD:containerMD">
			<xsl:element name="containerMD:container">
				<!-- Attributes -->
				<xsl:attribute name="creationDateTime">
					<xsl:value-of
						select="concat(substring-before(./j2:feature[@fid='http://jhove2.org/terms/property/org/jhove2/core/source/FileSource/LastModified']/j2:value,'+'),'Z')"
					/>
				</xsl:attribute>
				<!-- Elements -->
				<xsl:if
					test="number(./j2:feature[@fid='http://jhove2.org/terms/property/org/jhove2/core/source/FileSource/Size']/j2:value) &gt; 0">
					<xsl:element name="containerMD:fixity">
						<xsl:attribute name="size">
							<xsl:value-of
								select="./j2:feature[@fid='http://jhove2.org/terms/property/org/jhove2/core/source/FileSource/Size']/j2:value"
							/>
						</xsl:attribute>
					</xsl:element>
				</xsl:if>
				<xsl:element name="containerMD:formatDesignation">
					<xsl:element name="containerMD:formatName">
						<xsl:choose>
							<xsl:when test="exsl:node-set($GzipMemberSourceNodes)">
								<xsl:value-of
									select="exsl:node-set($GzipMemberSourceNodes)
							                 						/j2:features/j2:feature[@name='PresumptiveFormats']
										                            /j2:features/j2:feature[@name='PresumptiveFormat']
										                            /j2:features/j2:feature[@name='NativeIdentifier' and j2:features/j2:feature[@name='Namespace']/j2:value='MIME']
										                            /j2:features/j2:feature[@name='Value']
										                            /j2:value"
								/>
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of
									select="./j2:feature[@name='PresumptiveFormats']
							                              /j2:features/j2:feature[@name='PresumptiveFormat']
							                              /j2:features/j2:feature[@name='NativeIdentifier' and j2:features/j2:feature[@name='Namespace']/j2:value='MIME']
							                              /j2:features/j2:feature[@name='Value']
							                              /j2:value"
								/>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:element>
					<!-- L'élément formatVersion n'est créé que si cette information est disponible 20120104 -->
					<xsl:if test="exsl:node-set($VersionBlockNode)[@fid='http://jhove2.org/terms/property/org/jhove2/core/source/VersionBlockSource/VersionNumber']/j2:value">
						<xsl:element name="containerMD:formatVersion">
							<xsl:value-of
								select="exsl:node-set($VersionBlockNode)[@fid='http://jhove2.org/terms/property/org/jhove2/core/source/VersionBlockSource/VersionNumber']/j2:value"
							/>
						</xsl:element>
					</xsl:if>
				</xsl:element>
				<xsl:if test="exsl:node-set($GzipMemberSourceNodes)">
					<xsl:element name="containerMD:encoding">
						<!--  Atributtes -->
						<xsl:attribute name="type">
							<xsl:text>compression</xsl:text>
						</xsl:attribute>
						<xsl:attribute name="method">
							<xsl:value-of
								select="./j2:feature[@name='PresumptiveFormats']/j2:features/j2:feature[@name='PresumptiveFormat']/j2:features/j2:feature[@name='NativeIdentifier' and j2:features/j2:feature[@name='Namespace']/j2:value='MIME']/j2:features/j2:feature[@name='Value']/j2:value"
							/>
						</xsl:attribute>
						<xsl:if
							test="number(exsl:node-set($ArcModuleNode)[@fid='http://jhove2.org/terms/property/org/jhove2/module/format/arc/ArcModule/ArcFileSize']/j2:value) &gt; 0">
							<xsl:attribute name="originalSize">
								<xsl:value-of
									select="exsl:node-set($ArcModuleNode)[@fid='http://jhove2.org/terms/property/org/jhove2/module/format/arc/ArcModule/ArcFileSize']/j2:value"
								/>
							</xsl:attribute>
						</xsl:if>
					</xsl:element>
				</xsl:if>
				<xsl:element name="containerMD:assessmentInformation">
					<!-- Attributes -->
					<xsl:attribute name="agentName">
						<xsl:text>JHOVE2</xsl:text>
					</xsl:attribute>
					<xsl:attribute name="agentVersion">
						<xsl:value-of
							select="exsl:node-set($ArcModuleNode)[@fid='http://jhove2.org/terms/property/org/jhove2/module/Module/Version' and @fidns='JHOVE2']/j2:value/text()"
						/>
					</xsl:attribute>
					<!-- Elements -->
					<xsl:element name="containerMD:outcome">
						<xsl:choose>
							<xsl:when
								test="exsl:node-set($ArcModuleNode)[@fid='http://jhove2.org/terms/property/org/jhove2/module/format/Validator/isValid']/j2:value = 'true'">
								<xsl:text>Valid</xsl:text>
							</xsl:when>
							<xsl:otherwise>
								<xsl:text>Not-valid</xsl:text>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:element>
					<xsl:if
						test="exsl:node-set($ArcModuleNode)[@fid='http://jhove2.org/terms/property/org/jhove2/module/format/arc/ArcModule/Errors']/j2:value/text() != '{}'">
						<xsl:element name="containerMD:outcomeDetailNote">
							<xsl:value-of
								select="exsl:node-set($ArcModuleNode)[@fid='http://jhove2.org/terms/property/org/jhove2/module/format/arc/ArcModule/Errors']/j2:value/text()"
							/>
						</xsl:element>
					</xsl:if>
				</xsl:element>
				<!-- containerExtension est dans container -->
				<xsl:if test="$MimeType1='application/x-ia-arc' or $MimeType2='application/x-ia-arc' ">
					<xsl:call-template name="containerExtension"/>
				</xsl:if>
			</xsl:element>
			<!-- entries est au même niveau que container -->
			<xsl:if test="$MimeType1='application/x-ia-arc' or $MimeType2='application/x-ia-arc' ">
				<xsl:call-template name="entries"/>
			</xsl:if>
		</xsl:element>
	</xsl:template>
	<xsl:template name="containerExtension">
		<xsl:element name="containerMD:containerExtension">
			<xsl:element name="containerMD:ARCContainer">
				<xsl:element name="containerMD:fileName">
					<xsl:value-of
						select="exsl:node-set($VersionBlockNode)[@fid='http://jhove2.org/terms/property/org/jhove2/core/source/NamedSource/SourceName']/j2:value"
					/>
				</xsl:element>
				<xsl:element name="containerMD:profile">
					<xsl:value-of
						select="exsl:node-set($VersionBlockNode)[@fid='http://jhove2.org/terms/property/org/jhove2/core/source/VersionBlockSource/VersionNumber']/j2:value"/>
					<xsl:text>.</xsl:text>
					<xsl:value-of
						select="exsl:node-set($VersionBlockNode)[@fid='http://jhove2.org/terms/property/org/jhove2/core/source/VersionBlockSource/Reserved']/j2:value"
					/>
				</xsl:element>
				<xsl:element name="containerMD:originCode">
					<xsl:value-of
						select="exsl:node-set($VersionBlockNode)[@fid='http://jhove2.org/terms/property/org/jhove2/core/source/VersionBlockSource/OriginCode']/j2:value"
					/>
				</xsl:element>
				<xsl:element name="containerMD:versionBlockPayload">
					<xsl:choose>
						<xsl:when
							test="exsl:node-set($VersionBlockNode)[@name='PresumptiveFormats']
						/j2:features/j2:feature[@name='PresumptiveFormat']
						/j2:features/j2:feature[@name='NativeIdentifier']
						/j2:features[./j2:feature[@name='Namespace']/j2:value/text() ='MIME']
						/j2:feature[@name='Value']/j2:value/text() = 'application/xml' ">
							<xsl:text>true</xsl:text>
						</xsl:when>
						<xsl:otherwise>
							<xsl:text>false</xsl:text>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:element>
			</xsl:element>
		</xsl:element>
	</xsl:template>
	
	<xsl:template name="entries">
		<!-- ArcRecordSource -->
		<xsl:for-each
			select=".//j2:features/j2:feature[@ftid='http://jhove2.org/terms/reportable/org/jhove2/core/source/ArcRecordSource'
		and not(ancestor::j2:feature[@ftid='http://jhove2.org/terms/reportable/org/jhove2/core/source/ArcRecordSource'])]">
			<!-- Initialize variables -->
			<xsl:variable name="method"
				select="../../../j2:feature[@name='CompressionMethod']/j2:features/j2:feature[@name='Description']/j2:value"/>
			<xsl:variable name="size" select="./j2:features/j2:feature[@name='Length']"/>
			<xsl:variable name="dateTime" select="./j2:features/j2:feature[@name='RawArchiveDate']"/>
			<xsl:variable name="format"
				select="./j2:features/j2:feature[@name='PresumptiveFormats']/j2:features/j2:feature[@name='PresumptiveFormat']/j2:features/j2:feature[@name='NativeIdentifier' and j2:features/j2:feature[@name='Namespace']/j2:value='MIME']/j2:features/j2:feature[@name='Value']/j2:value/text()"/>
			<xsl:variable name="mimeType"
				select="./j2:features/j2:feature[@name='ContentType']/j2:value/text()"/>
			<xsl:variable name="sourceName"
				select="./j2:features/j2:feature[@name='SourceName']/j2:value/text()"/>
			<xsl:variable name="protocolVersion"
				select="./j2:features/j2:feature[@name='ProtocolVersion']/j2:value/text()"/>
			<xsl:variable name="codeResponse"
				select="./j2:features/j2:feature[@name='ProtocolResultCode']/j2:value"/>
			<!-- JAVA -->
			<xsl:value-of
				select="wrapper:addEntry($wrapperInstance,$sourceName,$size,$dateTime,$format,$mimeType,$protocolVersion,$codeResponse)"/>
			<xsl:if test="$method != ''">
				<xsl:value-of
					select="wrapper:handleEncoding($wrapperInstance,'compression',$method)"/>
			</xsl:if>
		</xsl:for-each>
		<xsl:if
			test="number(exsl:node-set($ArcModuleNode)[@fid='http://jhove2.org/terms/property/org/jhove2/module/format/arc/ArcModule/ArcRecordNumber']/j2:value) != 0">
			<xsl:element name="containerMD:entries">
				<xsl:element name="containerMD:entriesInformation">
					<xsl:attribute name="number">
						<xsl:value-of
							select="exsl:node-set($ArcModuleNode)[@fid='http://jhove2.org/terms/property/org/jhove2/module/format/arc/ArcModule/ArcRecordNumber']/j2:value"
						/>
					</xsl:attribute>
					<xsl:attribute name="minimumSize">
						<xsl:value-of select="wrapper:getMinimumSize($wrapperInstance)"/>
					</xsl:attribute>
					<xsl:attribute name="maximumSize">
						<xsl:value-of select="wrapper:getMaximumSize($wrapperInstance)"/>
					</xsl:attribute>
                    <xsl:if test="wrapper:getFirstDateTime($wrapperInstance) != ''">
						<xsl:attribute name="firstDateTime">
							<xsl:value-of select="wrapper:getFirstDateTime($wrapperInstance)"/>
						</xsl:attribute>
                    </xsl:if>
                    <xsl:if test="wrapper:getLastDateTime($wrapperInstance) != ''">
						<xsl:attribute name="lastDateTime">
							<xsl:value-of select="wrapper:getLastDateTime($wrapperInstance)"/>
						</xsl:attribute>
                    </xsl:if>
					<xsl:attribute name="globalSize">
						<xsl:value-of select="wrapper:getGlobalSize($wrapperInstance)"/>
					</xsl:attribute>
					<xsl:if test="wrapper:hasFormats($wrapperInstance) = 'true'">
						<xsl:element name="containerMD:formats">
							<xsl:value-of select="wrapper:getFormats($wrapperInstance)"
								disable-output-escaping="yes"/>
						</xsl:element>
					</xsl:if>
					<xsl:if test="wrapper:hasEncodings($wrapperInstance) = 'true'">
						<xsl:element name="containerMD:encodings">
							<xsl:value-of select="wrapper:getEncodings($wrapperInstance)"
								disable-output-escaping="yes"/>
						</xsl:element>
					</xsl:if>
					<xsl:element name="containerMD:entriesExtension">
						<xsl:element name="containerMD:ARCEntries">
							<xsl:if test="wrapper:hasDeclaredMimeTypes($wrapperInstance) = 'true'">
								<xsl:element name="containerMD:declaredMimeTypes">
									<xsl:value-of
										select="wrapper:getDeclaredMimeTypes($wrapperInstance)"
										disable-output-escaping="yes"/>
								</xsl:element>
							</xsl:if>
							<xsl:if test="wrapper:hasHosts($wrapperInstance) = 'true'">
								<xsl:element name="containerMD:hosts">
									<xsl:value-of select="wrapper:getHosts($wrapperInstance)"
										disable-output-escaping="yes"/>
								</xsl:element>
							</xsl:if>
							<xsl:if test="wrapper:hasResponses($wrapperInstance) = 'true'">
								<xsl:element name="containerMD:responses">
									<xsl:value-of select="wrapper:getResponses($wrapperInstance)"
										disable-output-escaping="yes"/>
								</xsl:element>
							</xsl:if>
						</xsl:element>
					</xsl:element>
				</xsl:element>
			</xsl:element>
		</xsl:if>
	</xsl:template>
</xsl:stylesheet>
